@@ -25,29 +25,41 @@
 #include "settings.h"
 #include "formlayoutattached.h"
 #include "mnemonicattached.h"
+#include "delegaterecycler.h"
+#include "scenepositionattached.h"
 
 #include <QQmlEngine>
 #include <QQmlContext>
 #include <QQuickItem>
 #include <QQuickStyle>
 
-#ifdef KIRIGAMI_BUILD_TYPE_STATIC
 #include "libkirigami/platformtheme.h"
-#else
-#include <platformtheme.h>
-#endif
 
 static QString s_selectedStyle;
 
+//Q_INIT_RESOURCE(kirigami);
+#ifdef KIRIGAMI_BUILD_TYPE_STATIC
+#include <qrc_kirigami.cpp>
+#endif
+
 QUrl KirigamiPlugin::componentUrl(const QString &fileName) const
 {
     foreach (const QString &style, m_stylesFallbackChain) {
         const QString candidate = QStringLiteral("styles/") + style + QLatin1Char('/') + fileName;
         if (QFile::exists(resolveFilePath(candidate))) {
+#ifdef KIRIGAMI_BUILD_TYPE_STATIC
+            return QUrl(QStringLiteral("qrc:/org/kde/kirigami/styles/") + style + QLatin1Char('/') + fileName);
+#else
             return QUrl(resolveFileUrl(candidate));
+#endif
         }
     }
+
+#ifdef KIRIGAMI_BUILD_TYPE_STATIC
+            return QUrl(QStringLiteral("qrc:/org/kde/kirigami/") + fileName);
+#else
     return QUrl(resolveFileUrl(fileName));
+#endif
 }
 
 
@@ -56,28 +68,33 @@ void KirigamiPlugin::registerTypes(const char *uri)
     Q_ASSERT(uri == QLatin1String("org.kde.kirigami"));
     const QString style = QQuickStyle::name();
 
+#if !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
     //org.kde.desktop.plasma is a couple of files that fall back to desktop by purpose
     if ((style.isEmpty() || style == QStringLiteral("org.kde.desktop.plasma")) && QFile::exists(resolveFilePath(QStringLiteral("/styles/org.kde.desktop")))) {
-#if !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
         m_stylesFallbackChain.prepend(QStringLiteral("org.kde.desktop"));
-#elif defined(Q_OS_ANDROID)
-        m_stylesFallbackChain.prepend(QStringLiteral("Material"));
-#else // do we have an iOS specific style?
-        m_stylesFallbackChain.prepend(QStringLiteral("Material"));
-#endif
     }
+#elif defined(Q_OS_ANDROID)
+    if (!m_stylesFallbackChain.contains(QStringLiteral("Material"))) {
+        m_stylesFallbackChain.prepend(QStringLiteral("Material"));
+    }
+#else // do we have an iOS specific style?
+    if (!m_stylesFallbackChain.contains(QStringLiteral("Material"))) {
+        m_stylesFallbackChain.prepend(QStringLiteral("Material"));
+    }
+#endif
 
-    if (!style.isEmpty() && QFile::exists(resolveFilePath(QStringLiteral("/styles/") + style))) {
+    if (!style.isEmpty() && QFile::exists(resolveFilePath(QStringLiteral("/styles/") + style)) && !m_stylesFallbackChain.contains(style)) {
         m_stylesFallbackChain.prepend(style);
         //if we have plasma deps installed, use them for extra integration
         if (style == QStringLiteral("org.kde.desktop") && QFile::exists(resolveFilePath(QStringLiteral("/styles/org.kde.desktop.plasma")))) {
             m_stylesFallbackChain.prepend("org.kde.desktop.plasma");
         }
     } else {
+#if !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
         m_stylesFallbackChain.prepend(QStringLiteral("org.kde.desktop"));
+#endif
     }
     //At this point the fallback chain will be selected->org.kde.desktop->Fallback
-
     s_selectedStyle = m_stylesFallbackChain.first();
 
     qmlRegisterSingletonType<Settings>(uri, 2, 0, "Settings",
@@ -110,17 +127,13 @@ void KirigamiPlugin::registerTypes(const char *uri)
     qmlRegisterType(componentUrl(QStringLiteral("Separator.qml")), uri, 2, 0, "Separator");
     qmlRegisterType(componentUrl(QStringLiteral("PageRow.qml")), uri, 2, 0, "PageRow");
 
-    //The icon is "special: we have to use a wrapper class to QIcon on org.kde.desktops
-#if !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
-    //we know that we want a different implementation with Material and Plasma styles
-    if (s_selectedStyle == QStringLiteral("Material") || s_selectedStyle == QStringLiteral("Plasma")) {
+    //we want a different implementation with Plasma style
+    if (s_selectedStyle == QStringLiteral("Plasma")) {
         qmlRegisterType(componentUrl(QStringLiteral("Icon.qml")), uri, 2, 0, "Icon");
     } else {
+        DesktopIcon::s_internalIconPath = resolveFilePath(QStringLiteral("icons"));
         qmlRegisterType<DesktopIcon>(uri, 2, 0, "Icon");
     }
-#else
-    qmlRegisterType(componentUrl(QStringLiteral("Icon.qml")), uri, 2, 0, "Icon");
-#endif
 
     qmlRegisterType(componentUrl(QStringLiteral("Label.qml")), uri, 2, 0, "Label");
     //TODO: uncomment for 2.3 release
@@ -146,6 +159,21 @@ void KirigamiPlugin::registerTypes(const char *uri)
     qmlRegisterUncreatableType<FormLayoutAttached>(uri, 2, 3, "FormData", "Cannot create objects of type FormData, use it as an attached poperty");
     qmlRegisterUncreatableType<MnemonicAttached>(uri, 2, 3, "MnemonicData", "Cannot create objects of type MnemonicData, use it as an attached poperty");
 
+    //2.4
+    qmlRegisterType(componentUrl(QStringLiteral("AbstractCard.qml")), uri, 2, 4, "AbstractCard");
+    qmlRegisterType(componentUrl(QStringLiteral("Card.qml")), uri, 2, 4, "Card");
+    qmlRegisterType(componentUrl(QStringLiteral("CardsListView.qml")), uri, 2, 4, "CardsListView");
+    qmlRegisterType(componentUrl(QStringLiteral("CardsGridView.qml")), uri, 2, 4, "CardsGridView");
+    qmlRegisterType(componentUrl(QStringLiteral("CardsLayout.qml")), uri, 2, 4, "CardsLayout");
+    qmlRegisterType(componentUrl(QStringLiteral("InlineMessage.qml")), uri, 2, 4, "InlineMessage");
+    qmlRegisterUncreatableType<MessageType>(uri, 2, 4, "MessageType", "Cannot create objects of type MessageType");
+    qmlRegisterType<DelegateRecycler>(uri, 2, 4, "DelegateRecycler");
+
+    //2.5
+    qmlRegisterType(componentUrl(QStringLiteral("ListItemDragHandle.qml")), uri, 2, 5, "ListItemDragHandle");
+    qmlRegisterType(componentUrl(QStringLiteral("ActionToolBar.qml")), uri, 2, 5, "ActionToolBar");
+    qmlRegisterUncreatableType<ScenePositionAttached>(uri, 2, 5, "ScenePosition", "Cannot create objects of type ScenePosition, use it as an attached poperty");
+
     qmlProtectModule(uri, 2);
 }
 
